@@ -10,7 +10,9 @@ from typing_extensions import TypedDict, Literal, Protocol
 
 from .. import build
 from .. import coredata
+from ..compilers import Compiler
 from ..mesonlib import MachineChoice, File, FileMode, FileOrString, OptionKey
+from ..modules.cmake import CMakeSubprojectOptions
 from ..programs import ExternalProgram
 
 
@@ -167,8 +169,8 @@ class RunTarget(TypedDict):
 class CustomTarget(TypedDict):
 
     build_always: bool
-    build_always_stale: bool
-    build_by_default: bool
+    build_always_stale: T.Optional[bool]
+    build_by_default: T.Optional[bool]
     capture: bool
     command: T.List[T.Union[str, build.BuildTarget, build.CustomTarget,
                             build.CustomTargetIndex, ExternalProgram, File]]
@@ -181,9 +183,9 @@ class CustomTarget(TypedDict):
     input: T.List[T.Union[str, build.BuildTarget, build.CustomTarget, build.CustomTargetIndex,
                           build.ExtractedObjects, build.GeneratedList, ExternalProgram, File]]
     install: bool
-    install_dir: T.List[T.Union[str, bool]]
+    install_dir: T.List[T.Union[str, T.Literal[False]]]
     install_mode: FileMode
-    install_tag: T.List[T.Union[str, bool]]
+    install_tag: T.List[T.Optional[str]]
     output: T.List[str]
     override_options: T.Dict[OptionKey, str]
 
@@ -239,3 +241,70 @@ class RunCommand(TypedDict):
     check: bool
     capture: T.Optional[bool]
     env: build.EnvironmentVariables
+
+
+class FeatureOptionRequire(TypedDict):
+
+    error_message: T.Optional[str]
+
+
+class DependencyPkgConfigVar(TypedDict):
+
+    default: T.Optional[str]
+    define_variable: T.List[str]
+
+
+class DependencyGetVariable(TypedDict):
+
+    cmake: T.Optional[str]
+    pkgconfig: T.Optional[str]
+    configtool: T.Optional[str]
+    internal: T.Optional[str]
+    default_value: T.Optional[str]
+    pkgconfig_define: T.List[str]
+
+
+class ConfigurationDataSet(TypedDict):
+
+    description: T.Optional[str]
+
+class VcsTag(TypedDict):
+
+    command: T.List[T.Union[str, build.BuildTarget, build.CustomTarget,
+                            build.CustomTargetIndex, ExternalProgram, File]]
+    fallback: T.Optional[str]
+    input: T.List[T.Union[str, build.BuildTarget, build.CustomTarget, build.CustomTargetIndex,
+                          build.ExtractedObjects, build.GeneratedList, ExternalProgram, File]]
+    output: T.List[str]
+    replace_string: str
+
+
+class ConfigureFile(TypedDict):
+
+    output: str
+    capture: bool
+    format: T.Literal['meson', 'cmake', 'cmake@']
+    output_format: T.Literal['c', 'nasm']
+    depfile: T.Optional[str]
+    install: T.Optional[bool]
+    install_dir: T.Union[str, T.Literal[False]]
+    install_mode: FileMode
+    install_tag: T.Optional[str]
+    encoding: str
+    command: T.Optional[T.List[T.Union[build.Executable, ExternalProgram, Compiler, File, str]]]
+    input: T.List[FileOrString]
+    configuration: T.Optional[T.Union[T.Dict[str, T.Union[str, int, bool]], build.ConfigurationData]]
+
+
+class Subproject(ExtractRequired):
+
+    default_options: T.List[str]
+    version: T.List[str]
+
+
+class DoSubproject(ExtractRequired):
+
+    default_options: T.List[str]
+    version: T.List[str]
+    cmake_options: T.List[str]
+    options: T.Optional[CMakeSubprojectOptions]
