@@ -288,11 +288,11 @@ class CompCertLinker(StaticLinker):
         return [f'-o{target}']
 
 
-class C2000Linker(StaticLinker):
+class TILinker(StaticLinker):
 
     def __init__(self, exelist: T.List[str]):
         super().__init__(exelist)
-        self.id = 'ar2000'
+        self.id = 'ti-ar'
 
     def can_linker_accept_rsp(self) -> bool:
         return False
@@ -304,6 +304,11 @@ class C2000Linker(StaticLinker):
         return ['-r']
 
 
+class C2000Linker(TILinker):
+    # Required for backwards compat with projects created before ti-cgt support existed
+    id = 'ar2000'
+
+
 class AIXArLinker(ArLikeLinker):
     id = 'aixar'
     std_args = ['-csr', '-Xany']
@@ -818,7 +823,8 @@ class LLVMDynamicLinker(GnuLikeDynamicLinkerMixin, PosixDynamicLinkerMixin, Dyna
 
         # Some targets don't seem to support this argument (windows, wasm, ...)
         _, _, e = mesonlib.Popen_safe(self.exelist + self._apply_prefix('--allow-shlib-undefined'))
-        self.has_allow_shlib_undefined = 'unknown argument: --allow-shlib-undefined' not in e
+        # Versions < 9 do not have a quoted argument
+        self.has_allow_shlib_undefined = ('unknown argument: --allow-shlib-undefined' not in e) and ("unknown argument: '--allow-shlib-undefined'" not in e)
 
     def get_allow_undefined_args(self) -> T.List[str]:
         if self.has_allow_shlib_undefined:
@@ -971,15 +977,15 @@ class CompCertDynamicLinker(DynamicLinker):
                          install_rpath: str) -> T.Tuple[T.List[str], T.Set[bytes]]:
         return ([], set())
 
-class C2000DynamicLinker(DynamicLinker):
+class TIDynamicLinker(DynamicLinker):
 
-    """Linker for Texas Instruments C2000 compiler."""
+    """Linker for Texas Instruments compiler family."""
 
-    id = 'cl2000'
+    id = 'ti'
 
     def __init__(self, exelist: T.List[str], for_machine: mesonlib.MachineChoice,
                  *, version: str = 'unknown version'):
-        super().__init__(exelist or ['cl2000.exe'], for_machine, '', [],
+        super().__init__(exelist, for_machine, '', [],
                          version=version)
 
     def get_link_whole_for(self, args: T.List[str]) -> T.List[str]:
@@ -1000,7 +1006,7 @@ class C2000DynamicLinker(DynamicLinker):
         return ['-z', f'--output_file={outputname}']
 
     def get_search_args(self, dirname: str) -> 'T.NoReturn':
-        raise OSError('cl2000.exe does not have a search dir argument')
+        raise OSError('TI compilers do not have a search dir argument')
 
     def get_allow_undefined_args(self) -> T.List[str]:
         return []
@@ -1009,6 +1015,11 @@ class C2000DynamicLinker(DynamicLinker):
         return []
 
 
+class C2000DynamicLinker(TIDynamicLinker):
+    # Required for backwards compat with projects created before ti-cgt support existed
+    id = 'cl2000'
+
+
 class ArmDynamicLinker(PosixDynamicLinkerMixin, DynamicLinker):
 
     """Linker for the ARM compiler."""
@@ -1172,7 +1183,7 @@ class VisualStudioLikeLinkerMixin:
 
     def get_always_args(self) -> T.List[str]:
         parent = super().get_always_args() # type: ignore
-        return self._apply_prefix('/nologo') + T.cast(T.List[str], parent)
+        return self._apply_prefix('/nologo') + T.cast('T.List[str]', parent)
 
     def get_search_args(self, dirname: str) -> T.List[str]:
         return self._apply_prefix('/LIBPATH:' + dirname)
