@@ -38,15 +38,44 @@ from ..mesonlib import MachineChoice
 from ..programs import ExternalProgram, NonExistingExternalProgram
 
 if T.TYPE_CHECKING:
+    from typing_extensions import TypedDict
+
     from . import ModuleState
     from ..build import SharedModule, Data
     from ..dependencies import ExternalDependency, Dependency
     from ..dependencies.factory import DependencyGenerator
     from ..environment import Environment
     from ..interpreter import Interpreter
+    from ..interpreter.kwargs import ExtractRequired
     from ..interpreterbase.interpreterbase import TYPE_var, TYPE_kwargs
 
-    from typing_extensions import TypedDict
+    class PythonIntrospectionDict(TypedDict):
+
+        install_paths: T.Dict[str, str]
+        is_pypy: bool
+        is_venv: bool
+        link_libpython: bool
+        sysconfig_paths: T.Dict[str, str]
+        paths: T.Dict[str, str]
+        platform: str
+        suffix: str
+        variables: T.Dict[str, str]
+        version: str
+
+    class PyInstallKw(TypedDict):
+
+        pure: bool
+        subdir: str
+        install_tag: T.Optional[str]
+
+    class FindInstallationKw(ExtractRequired):
+
+        disabler: bool
+        modules: T.List[str]
+
+    _Base = ExternalDependency
+else:
+    _Base = object
 
 
 mod_kwargs = {'subdir'}
@@ -54,11 +83,6 @@ mod_kwargs.update(known_shmod_kwargs)
 mod_kwargs -= {'name_prefix', 'name_suffix'}
 
 
-if T.TYPE_CHECKING:
-    _Base = ExternalDependency
-else:
-    _Base = object
-
 class _PythonDependencyBase(_Base):
 
     def __init__(self, python_holder: 'PythonInstallation', embed: bool):
@@ -79,10 +103,19 @@ class _PythonDependencyBase(_Base):
 class PythonPkgConfigDependency(PkgConfigDependency, _PythonDependencyBase):
 
     def __init__(self, name: str, environment: 'Environment',
-                 kwargs: T.Dict[str, T.Any], installation: 'PythonInstallation'):
+                 kwargs: T.Dict[str, T.Any], installation: 'PythonInstallation',
+                 libpc: bool = False):
+        if libpc:
+            mlog.debug(f'Searching for {name!r} via pkgconfig lookup in LIBPC')
+        else:
+            mlog.debug(f'Searching for {name!r} via fallback pkgconfig lookup in default paths')
+
         PkgConfigDependency.__init__(self, name, environment, kwargs)
         _PythonDependencyBase.__init__(self, installation, kwargs.get('embed', False))
 
+        if libpc and not self.is_found:
+            mlog.debug(f'"python-{self.version}" could not be found in LIBPC, this is likely due to a relocated python installation')
+
 
 class PythonFrameworkDependency(ExtraFrameworkDependency, _PythonDependencyBase):
 
@@ -124,7 +157,7 @@ class PythonSystemDependency(SystemDependency, _PythonDependencyBase):
         if largs is not None:
             self.link_args = largs
 
-        self.is_found = largs is not None or self.link_libpython
+        self.is_found = largs is not None or not self.link_libpython
 
         inc_paths = mesonlib.OrderedSet([
             self.variables.get('INCLUDEPY'),
@@ -240,12 +273,15 @@ def python_factory(env: 'Environment', for_machine: 'MachineChoice',
         # If python-X.Y.pc exists in LIBPC, we will try to use it
         def wrap_in_pythons_pc_dir(name: str, env: 'Environment', kwargs: T.Dict[str, T.Any],
                                    installation: 'PythonInstallation') -> 'ExternalDependency':
+            if not pkg_libdir:
+                # there is no LIBPC, so we can't search in it
+                return NotFoundDependency('python', env)
+
             old_pkg_libdir = os.environ.pop('PKG_CONFIG_LIBDIR', None)
             old_pkg_path = os.environ.pop('PKG_CONFIG_PATH', None)
-            if pkg_libdir:
-                os.environ['PKG_CONFIG_LIBDIR'] = pkg_libdir
+            os.environ['PKG_CONFIG_LIBDIR'] = pkg_libdir
             try:
-                return PythonPkgConfigDependency(name, env, kwargs, installation)
+                return PythonPkgConfigDependency(name, env, kwargs, installation, True)
             finally:
                 def set_env(name, value):
                     if value is not None:
@@ -255,10 +291,11 @@ def python_factory(env: 'Environment', for_machine: 'MachineChoice',
                 set_env('PKG_CONFIG_LIBDIR', old_pkg_libdir)
                 set_env('PKG_CONFIG_PATH', old_pkg_path)
 
-        candidates.extend([
-            functools.partial(wrap_in_pythons_pc_dir, pkg_name, env, kwargs, installation),
-            functools.partial(PythonPkgConfigDependency, pkg_name, env, kwargs, installation)
-        ])
+        candidates.append(functools.partial(wrap_in_pythons_pc_dir, pkg_name, env, kwargs, installation))
+        # We only need to check both, if a python install has a LIBPC. It might point to the wrong location,
+        # e.g. relocated / cross compilation, but the presence of LIBPC indicates we should definitely look for something.
+        if pkg_libdir is not None:
+            candidates.append(functools.partial(PythonPkgConfigDependency, pkg_name, env, kwargs, installation))
 
     if DependencyMethods.SYSTEM in methods:
         candidates.append(functools.partial(PythonSystemDependency, 'python', env, kwargs, installation))
@@ -325,27 +362,16 @@ variables.update({'base_prefix': getattr(sys, 'base_prefix', sys.prefix)})
 print(json.dumps({
   'variables': variables,
   'paths': paths,
+  'sysconfig_paths': sysconfig.get_paths(),
   'install_paths': install_paths,
-  'sys_paths': sys.path,
   'version': sysconfig.get_python_version(),
   'platform': sysconfig.get_platform(),
   'is_pypy': '__pypy__' in sys.builtin_module_names,
+  'is_venv': sys.prefix != variables['base_prefix'],
   'link_libpython': links_against_libpython(),
 }))
 '''
 
-if T.TYPE_CHECKING:
-    class PythonIntrospectionDict(TypedDict):
-
-        install_paths: T.Dict[str, str]
-        is_pypy: bool
-        link_libpython: bool
-        paths: T.Dict[str, str]
-        platform: str
-        suffix: str
-        variables: T.Dict[str, str]
-        version: str
-
 
 class PythonExternalProgram(ExternalProgram):
     def __init__(self, name: str, command: T.Optional[T.List[str]] = None,
@@ -364,7 +390,9 @@ class PythonExternalProgram(ExternalProgram):
         self.info: 'PythonIntrospectionDict' = {
             'install_paths': {},
             'is_pypy': False,
+            'is_venv': False,
             'link_libpython': False,
+            'sysconfig_paths': {},
             'paths': {},
             'platform': 'sentinal',
             'variables': {},
@@ -380,8 +408,13 @@ class PythonExternalProgram(ExternalProgram):
 
     def sanity(self, state: T.Optional['ModuleState'] = None) -> bool:
         # Sanity check, we expect to have something that at least quacks in tune
-        cmd = self.get_command() + ['-c', INTROSPECT_COMMAND]
+        from tempfile import NamedTemporaryFile
+        with NamedTemporaryFile(suffix='.py', delete=False, mode='w', encoding='utf-8') as tf:
+            tmpfilename = tf.name
+            tf.write(INTROSPECT_COMMAND)
+        cmd = self.get_command() + [tmpfilename]
         p, stdout, stderr = mesonlib.Popen_safe(cmd)
+        os.unlink(tmpfilename)
         try:
             info = json.loads(stdout)
         except json.JSONDecodeError:
@@ -409,29 +442,28 @@ class PythonExternalProgram(ExternalProgram):
             return rel_path
         value = state.get_option(f'{key}dir', module='python')
         if value:
+            if state.is_user_defined_option('install_env', module='python'):
+                raise mesonlib.MesonException(f'python.{key}dir and python.install_env are mutually exclusive')
             return value
-        # Use python's path relative to prefix, and warn if that's not a location
-        # python will lookup for modules.
-        abs_path = Path(state.get_option('prefix'), rel_path)
-        sys_paths = [Path(i) for i in self.info['sys_paths']]
-        if abs_path not in sys_paths:
-            mlog.warning('Python files installed by Meson might not be found by python interpreter.\n',
-                         f'This warning can be avoided by setting "python.{key}dir" option.',
-                         once=True)
+
+        install_env = state.get_option('install_env', module='python')
+        if install_env == 'auto':
+            install_env = 'venv' if self.info['is_venv'] else 'system'
+
+        if install_env == 'system':
+            rel_path = os.path.join(self.info['variables']['prefix'], rel_path)
+        elif install_env == 'venv':
+            if not self.info['is_venv']:
+                raise mesonlib.MesonException('python.install_env cannot be set to "venv" unless you are in a venv!')
+            # inside a venv, deb_system is *never* active hence info['paths'] may be wrong
+            rel_path = self.info['sysconfig_paths'][key]
+
         return rel_path
 
 
 _PURE_KW = KwargInfo('pure', bool, default=True)
 _SUBDIR_KW = KwargInfo('subdir', str, default='')
 
-if T.TYPE_CHECKING:
-
-    class PyInstallKw(TypedDict):
-
-        pure: bool
-        subdir: str
-        install_tag: T.Optional[str]
-
 
 class PythonInstallation(ExternalProgramHolder):
     def __init__(self, python: 'PythonExternalProgram', interpreter: 'Interpreter'):
@@ -589,20 +621,12 @@ class PythonInstallation(ExternalProgramHolder):
         return super().path_method(args, kwargs)
 
 
-if T.TYPE_CHECKING:
-    from ..interpreter.kwargs import ExtractRequired
-
-    class FindInstallationKw(ExtractRequired):
-
-        disabler: bool
-        modules: T.List[str]
-
-
 class PythonModule(ExtensionModule):
 
     @FeatureNew('Python Module', '0.46.0')
     def __init__(self, interpreter: 'Interpreter') -> None:
         super().__init__(interpreter)
+        self.installations: T.Dict[str, ExternalProgram] = {}
         self.methods.update({
             'find_installation': self.find_installation,
         })
@@ -624,39 +648,7 @@ class PythonModule(ExtensionModule):
         else:
             return None
 
-    @disablerIfNotFound
-    @typed_pos_args('python.find_installation', optargs=[str])
-    @typed_kwargs(
-        'python.find_installation',
-        KwargInfo('required', (bool, UserFeatureOption), default=True),
-        KwargInfo('disabler', bool, default=False, since='0.49.0'),
-        KwargInfo('modules', ContainerTypeInfo(list, str), listify=True, default=[], since='0.51.0'),
-    )
-    def find_installation(self, state: 'ModuleState', args: T.Tuple[T.Optional[str]],
-                          kwargs: 'FindInstallationKw') -> ExternalProgram:
-        feature_check = FeatureNew('Passing "feature" option to find_installation', '0.48.0')
-        disabled, required, feature = extract_required_kwarg(kwargs, state.subproject, feature_check)
-        want_modules = kwargs['modules']
-        found_modules: T.List[str] = []
-        missing_modules: T.List[str] = []
-
-        # FIXME: this code is *full* of sharp corners. It assumes that it's
-        # going to get a string value (or now a list of length 1), of `python2`
-        # or `python3` which is completely nonsense.  On windows the value could
-        # easily be `['py', '-3']`, or `['py', '-3.7']` to get a very specific
-        # version of python. On Linux we might want a python that's not in
-        # $PATH, or that uses a wrapper of some kind.
-        np: T.List[str] = state.environment.lookup_binary_entry(MachineChoice.HOST, 'python') or []
-        fallback = args[0]
-        display_name = fallback or 'python'
-        if not np and fallback is not None:
-            np = [fallback]
-        name_or_path = np[0] if np else None
-
-        if disabled:
-            mlog.log('Program', name_or_path or 'python', 'found:', mlog.red('NO'), '(disabled by:', mlog.bold(feature), ')')
-            return NonExistingExternalProgram()
-
+    def _find_installation_impl(self, state: 'ModuleState', display_name: str, name_or_path: str, required: bool) -> ExternalProgram:
         if not name_or_path:
             python = PythonExternalProgram('python3', mesonlib.python_command)
         else:
@@ -676,6 +668,56 @@ class PythonModule(ExtensionModule):
             if not python.found() and name_or_path in ['python2', 'python3']:
                 python = PythonExternalProgram('python')
 
+        if python.found():
+            if python.sanity(state):
+                return python
+            else:
+                sanitymsg = f'{python} is not a valid python or it is missing distutils'
+                if required:
+                    raise mesonlib.MesonException(sanitymsg)
+                else:
+                    mlog.warning(sanitymsg, location=state.current_node)
+
+        return NonExistingExternalProgram()
+
+    @disablerIfNotFound
+    @typed_pos_args('python.find_installation', optargs=[str])
+    @typed_kwargs(
+        'python.find_installation',
+        KwargInfo('required', (bool, UserFeatureOption), default=True),
+        KwargInfo('disabler', bool, default=False, since='0.49.0'),
+        KwargInfo('modules', ContainerTypeInfo(list, str), listify=True, default=[], since='0.51.0'),
+    )
+    def find_installation(self, state: 'ModuleState', args: T.Tuple[T.Optional[str]],
+                          kwargs: 'FindInstallationKw') -> ExternalProgram:
+        feature_check = FeatureNew('Passing "feature" option to find_installation', '0.48.0')
+        disabled, required, feature = extract_required_kwarg(kwargs, state.subproject, feature_check)
+
+        # FIXME: this code is *full* of sharp corners. It assumes that it's
+        # going to get a string value (or now a list of length 1), of `python2`
+        # or `python3` which is completely nonsense.  On windows the value could
+        # easily be `['py', '-3']`, or `['py', '-3.7']` to get a very specific
+        # version of python. On Linux we might want a python that's not in
+        # $PATH, or that uses a wrapper of some kind.
+        np: T.List[str] = state.environment.lookup_binary_entry(MachineChoice.HOST, 'python') or []
+        fallback = args[0]
+        display_name = fallback or 'python'
+        if not np and fallback is not None:
+            np = [fallback]
+        name_or_path = np[0] if np else None
+
+        if disabled:
+            mlog.log('Program', name_or_path or 'python', 'found:', mlog.red('NO'), '(disabled by:', mlog.bold(feature), ')')
+            return NonExistingExternalProgram()
+
+        python = self.installations.get(name_or_path)
+        if not python:
+            python = self._find_installation_impl(state, display_name, name_or_path, required)
+            self.installations[name_or_path] = python
+
+        want_modules = kwargs['modules']
+        found_modules: T.List[str] = []
+        missing_modules: T.List[str] = []
         if python.found() and want_modules:
             for mod in want_modules:
                 p, *_ = mesonlib.Popen_safe(
@@ -709,14 +751,7 @@ class PythonModule(ExtensionModule):
                 raise mesonlib.MesonException('{} is missing modules: {}'.format(name_or_path or 'python', ', '.join(missing_modules)))
             return NonExistingExternalProgram()
         else:
-            sane = python.sanity(state)
-
-            if sane:
-                return python
-            else:
-                if required:
-                    raise mesonlib.MesonException(f'{python} is not a valid python or it is missing distutils')
-                return NonExistingExternalProgram()
+            return python
 
         raise mesonlib.MesonBugException('Unreachable code was reached (PythonModule.find_installation).')
 
